unit DirFuncts;

interface

uses Windows,SysUtils,Classes, contnrs;

type

  TDirList = class(TObjectList)
  protected
    FOwner:TDirList;
    function getFullPath:string;
  public
    SearchRec:TSearchRec;
    constructor Create(Owner:TDirList);
    procedure WriteToStream(Stream:TStream;k:integer);
    destructor Destroy;override;
    property FullPath:string read getFullPath;
  end;

  TFileAttribs = class
  public
    SearchRec:TSearchRec;
    constructor Create(ASearchRec:TSearchRec);
    procedure WriteToStream(Stream:TStream;k:integer);
  end;

  RFileAttribs = record
    Size: Integer;
    Attr: Integer;
    ftCreationTime: TDateTime;
    ftLastAccessTime: TDateTime;
    ftLastWriteTime: TDateTime;
  end;

  RDirTree = record
    Size: Integer;
    Attr: Integer;
    ftCreationTime: TDateTime;
    ftLastAccessTime: TDateTime;
    ftLastWriteTime: TDateTime;
    Files:TStringList;
  End;

  TFileEvent = procedure (Sender:TObject;SearchRec:TSearchRec;DirList:TDirList;var Break:boolean) of object;

  TDirectory = class
  private
    { Private declarations }
  protected
    { Private declarations }
    FOnFindFile:TFileEvent;
  public
    { Public declarations }
    function ProcessDir(DirList:TDirList;const FileNames:TFileName;const AAttr:integer=faAnyFile):integer;
  published
    property OnFindFile:TFileEvent read FOnFindFile write FOnFindFile;
  end;

implementation

function TDirectory.ProcessDir(DirList:TDirList; const FileNames:TFileName; const AAttr:integer=faAnyFile):integer;
var lvBreak:boolean;
    SearchRec:TSearchRec;
begin
  Assert(Assigned(FOnFindFile));
  lvBreak:=false;

  Result := FindFirst(DirList.FullPath+'\'+FileNames, AAttr, SearchRec);
  while not lvBreak and (Result = 0) do
  begin
    FOnFindFile(Self,SearchRec,DirList,lvBreak);
    Result := FindNext(SearchRec);
  end;

  FindClose(SearchRec);
end;


{ TFileAttribs }

constructor TFileAttribs.Create(ASearchRec: TSearchRec);
begin
  SearchRec:=ASearchRec;
end;

procedure TFileAttribs.WriteToStream(Stream: TStream; k: integer);
var s:string;
begin
  s:=StringOfChar(' ',k)+SearchRec.Name+' ['+intToStr(SearchRec.Size)+']'#13#10;
  Stream.WriteBuffer(Pointer(s)^,length(s));
end;

{ TDirList }

constructor TDirList.Create(Owner: TDirList);
begin
  inherited Create;
  FOwner:=Owner;
end;

destructor TDirList.Destroy;
var cv:integer;
begin
  for cv:=0 to Count-1 do
    Items[cv].Free;
end;

function TDirList.getFullPath: string;
begin
  if assigned(FOwner) then
    result:=FOwner.FullPath+'\'+SearchRec.Name
  else
    result:=SearchRec.Name;
end;

procedure TDirList.WriteToStream(Stream:TStream;k:integer);
var s:string;
   cv:integer;
begin
  s:=StringOfChar(' ',k)+SearchRec.Name+#13#10;
  Stream.WriteBuffer(Pointer(s)^,length(s));
  for cv:=0 to Count-1 do
  begin
    if Items[cv] is TDirList then
      TDirList(Items[cv]).WriteToStream(Stream,k+2)
    else
      TFileAttribs(Items[cv]).WriteToStream(Stream,k+2)
  end;
end;

end.
