unit UzStringParser;

interface
uses Classes, strfconst;

type
  TzStringParser = class
  private
    FDelims:TDelimGroup;
    FFCharPos:integer;
    FLength:integer;
    FString:string;
    FMakeDif:boolean;
    FEndPos:integer;
    FCaseInsensitive:boolean;
    FDelimIndex:integer;
  protected
    function getLastCharPos: integer;
    procedure setLastCharPos(const Value:integer);
    procedure setEndPos(const Value:integer);
    procedure setTokenLength(const Value:integer);
    procedure setInputString(const Value:String);
    function getAsString:String;
    procedure setAsString(const Value:String);
    procedure setFirstCharPos(const Value:integer);
  public
    constructor Create();
    procedure setDelims(ADelims:TDelimGroup);
    procedure AddDelim(ADelim:TDelims);
    function  GetDelim(AIndex:integer):TDelims;
    procedure RemoveDelim(AIndex:integer);
    function  IndexOfDelim(ADelim:TDelims):integer;
    function  GetNextToken(const AFromPos:integer; const ADirection:TDirection):boolean;
    function  GetNextNToken(AN:Integer; const ADirection:TDirection; AFromPos:integer=-1):integer;
    function  PosFrom(ASubString:string;const AJokerChar:char='?';const AAsteriskChar:char='*'):integer;
    property  EndPos:integer read FEndPos write FEndPos;
    property  InputString:string read FString write setInputString;
    property  asString:string read getAsString write setAsString;
    property  FCharPos:integer read FFCharPos write setFirstCharPos;
    property  Len:integer read FLength write setTokenLength;
    property  LCharPos:integer read getLastCharPos write setLastCharPos;
    property  MakeDif:boolean read FMakeDif write FMakeDif;
    property  DelimIndex:integer read FDelimIndex write FDelimIndex;
  End;

implementation

uses sysutils;

constructor TzStringParser.Create();
Begin
  FFCharPos:=1;
  FLength:= 0;
  FDelimIndex:=-1;
  Initialize(FDelims,0);
End;

function TzStringParser.GetNextToken(const AFromPos:integer; const ADirection:TDirection):boolean;
Begin
  Result:=True;
  try
    if MakeDif then
    Begin
      FDelimIndex:=0;
      if not (FString[AFromPos] in FDelims[0]) then
        while not(FString[AFromPos] in FDelims[FDelimIndex]) and (FDelimIndex<Length(FDelims)) do
          inc(FDelimIndex);
    End;

    Assert((AFromPos<=Length(FString)) and (AFromPos>0));

    FFCharPos:=AFromPos;

    if ADirection in [dirMiddle,dirLeft] then
    Begin
      if not(FMakeDif xor (FString[FFCharPos] in FDelims[FDelimIndex])) then
        try
          While (FFCharPos>1) and not(FMakeDif xor (FString[FFCharPos-1] in FDelims[FDelimIndex])) do
            dec(FFCharPos)
        except
        end
    End;

    if ADirection in [dirMiddle,dirRight] then
    Begin
      FLength:=1;
      if not(FMakeDif xor (FString[AFromPos] in FDelims[FDelimIndex])) then
        try
          While not(FMakeDif xor (FString[AFromPos+FLength] in FDelims[FDelimIndex])) do
            inc(FLength);
        except
        end;
        FLength:=FLength+(AFromPos-FFCharPos);
    end
    else
      FLength:=(AFromPos-FFCharPos)+1;

  except
    FLength:=0;
    Result:=False;
  end;
End;

function TzStringParser.GetNextNToken(AN:Integer; const ADirection:TDirection; AFromPos:integer=-1):integer;
Begin
  if AFromPos=-1 then
    AFromPos:=FFCharPos;
  if GetNextToken(AFromPos,ADirection)then
  Begin
    dec(AN);
    if ADirection = dirLeft then
      while (AN>0) and GetNextToken(FFCharPos-1,ADirection) do
        dec(AN)
    else
    if ADirection = dirRight then
      while (AN>0) and GetNextToken(LCharPos+1,ADirection) do
        dec(AN)
  End;
  Result:=AN;
End;

function TzStringParser.getAsString:String;
Begin
  Result:=Copy(FString,FFCharPos,FLength);
  if FLength<>Length(Result) then
    FLength:=Length(Result);
End;

procedure TzStringParser.setInputString(const Value:String);
Begin
  FString:=Value;
  FFCharPos:=1;
  FLength:=0;
  FEndPos:=Length(FString);
End;

procedure TzStringParser.setAsString(const Value:String);
Begin
  Delete(FString,FFCharPos,FLength);
  Insert(Value,FString,FFCharPos);
  FLength:=Length(Value);
  if FEndPos>Length(FString) then
    FEndPos:=Length(FString);
End;

function TzStringParser.getLastCharPos: integer;
begin
  if FLength=0 then
    Result:=0
  else
    Result:=FFCharPos+FLength-1;
end;

procedure TzStringParser.setFirstCharPos(const Value: integer);
begin
  FFCharPos:=Value;
end;

procedure TzStringParser.setLastCharPos(const Value: integer);
begin
  if (Value>=FFCharPos) and (Value<=Length(FString)) then
    FLength:=Value-FFCharPos+1;
end;

procedure TzStringParser.setTokenLength(const Value: integer);
begin
  FLength := Value;
end;

procedure TzStringParser.setDelims(ADelims:TDelimGroup);
begin
  FDelims:=Copy(ADelims,0,Length(ADelims));
end;

procedure TzStringParser.AddDelim(ADelim: TDelims);
var lvlast:integer;
begin
  lvlast:=Length(FDelims);
  SetLength(FDelims,lvlast+1);
  FDelims[lvlast]:=ADelim;
end;

function TzStringParser.GetDelim(AIndex: integer): TDelims;
begin
  Result:=FDelims[AIndex];
end;

function TzStringParser.IndexOfDelim(ADelim: TDelims): integer;
begin
  Result:=0;
  try
    while FDelims[Result]<>ADelim do
      inc(Result);
  except
    Result:=-1;
  end
end;

procedure TzStringParser.RemoveDelim(AIndex: integer);
var cv:integer;
begin
  cv:=AIndex;
  while cv<High(FDelims)-1 do
    FDelims[cv]:=FDelims[cv+1];
  SetLength(FDelims,cv+1);
end;

function TzStringParser.PosFrom(ASubString: string; const AJokerChar, AAsteriskChar: char): integer;
var cv,cv1,cv2,lvStrLen,lvSubLen,lvSubStart,lvTempStart,lvEnd,lvFoundLength:integer;
    lvFound,lvExit:boolean;
    lvCharA,lvCharB:char;
begin
  Result:=-1;
  FLength:=0;
  lvFoundLength:=0;
  lvSubStart:=0;
  lvStrLen:=length(FString);
  lvSubLen:=length(ASubString);
  lvEnd:=FEndPos;

  if (FFCharPos<1) or (FFCharPos>lvEnd) or (lvEnd>lvStrLen) or
     (FFCharPos>lvStrLen) then
    exit;

  lvTempStart:=1;
  while (ASubString[lvTempStart]=AAsteriskChar) and (lvTempStart<lvSubLen) do
    inc(lvTempStart);
  if (lvTempStart=lvSubLen) and (ASubString[lvTempStart]=AAsteriskChar) then
    exit
  else
  if lvTempStart>1 then
  Begin
    ASubString:=Copy(ASubString,lvTempStart,lvSubLen-lvTempStart+1);
    lvSubLen:=length(ASubString);
  End;

  cv:=FFCharPos;
  lvExit:=False;
  while (cv<=lvEnd) and (not lvExit) do
  Begin
    lvCharA:=FString[cv];
    lvCharB:=ASubString[1];
    if FCaseInSensitive then
    Begin
      lvCharA:=HuLowChar(lvCharA);
      lvCharB:=HuLowChar(lvCharB);
    End;
    if (lvCharA in FDelims[FDelimIndex]) then
      break;
    if (lvCharA=lvCharB) or (lvCharB=AJokerChar) then
    Begin
      cv1:=1;
      lvFound:=true;
      Repeat
        if ((cv+cv1)>lvEnd) then
        Begin
          if (lvSubLen>cv1) and not(((cv1+1)=lvSubLen) and (ASubString[cv1+1]=AAsteriskChar)) then
            lvFound:=False;
          break;
        End;
        if (cv1+1)>lvSubLen then
          break;
        lvCharA:=FString[cv+cv1];
        lvCharB:=ASubString[cv1+1];
        if FCaseInSensitive then
        Begin
          lvCharA:=HuLowChar(lvCharA);
          lvCharB:=HuLowChar(lvCharB);
        End;
        if lvCharB=AAsteriskChar then
        Begin
          if (cv+cv1=lvStrLen) then
          Begin
            lvCharB:=ASubString[cv1+2];
            if FCaseInSensitive then
              lvCharB:=HuLowChar(lvCharB);
            if ((cv1+1)<lvSubLen) and
              not (  ( ((cv1+2)=lvSubLen) and (lvCharA=lvCharB) )
                   or ( ((cv1+3)=lvSubLen) and (ASubString[cv1+3]=AAsteriskChar) )  ) then
                lvFound:=False;
            inc(cv1);
            Break;
          End;
          if (cv1+1=lvSubLen) then
          Begin
            cv2:=cv+cv1;
            while (not(FString[cv2] in FDelims[FDelimIndex])) do
            Begin
              inc(cv2);
              if cv2>lvStrLen then
                break;
            End;
            lvFoundLength:=cv2-cv;
            lvSubStart:=cv;
            Break;
          End;

          lvSubStart:=PosFrom(copy(ASubString,cv1+2,lvSubLen),AJokerChar,AAsteriskChar);
          if lvSubStart>-1 then
            break
          else
          Begin
            lvFound:=False;
            lvExit:=True;
            break;
          End;
        End;
        if (lvCharA in FDelims[FDelimIndex]) then
          break;
        if ((lvCharA<>lvCharB) and (lvCharB<>AJokerChar)) or (lvCharA in FDelims[FDelimIndex]) then
        Begin
          lvFound:=False;
          break;
        End;
        inc(cv1);
      until ((cv1+1)>lvSubLen);
      if lvFound then
      Begin
        Result:=cv;
        if lvFoundLength=0 then
          FLength:=cv1
        else
          FLength:=lvSubStart-Result+lvFoundLength;
        break
      End;
      if ((cv+cv1)>lvEnd) or (FString[cv+cv1] in FDelims[FDelimIndex]) then
        break;
    End;
    inc(cv);
  End;
end;

procedure TzStringParser.setEndPos(const Value: integer);
begin
  if Value<=Length(FString) then
    FEndPos:=Value
  else
    raise ERangeError.Create('EndPos out of range');
end;

end.
