unit strfconst;

interface

uses contnrs, Math;

type

  TDirection = (dirLeft,dirMiddle,dirRight);

  TDelims = set of char;

  TDelimGroup = array of set of char;

  TSpecialStrings = array of string;

  TIntPairs = record
    A:integer;
    B:integer;
  end;

  TIntPairArray = array of TIntPairs;

  TStringPairs = record
    A:String;
    B:String;
  end;

  TStringPairArray = array of TStringPairs;

  TCustomSemanticalObjectClass = class of TCustomSemanticalObject;

  TCustomSemanticalObject = class
  protected
    FFitDirection:integer;
    FMask:String;
    procedure SetStrValue(Value:String);virtual;abstract;
    function GetStrValue:String;virtual;abstract;
    function GetFitPercent:real;virtual;abstract;
    procedure SetFitPercent(Value:real);virtual;abstract;
  published
    function FitChar(Value:Char):boolean;virtual;abstract;
    property FitDirection:integer read FFitDirection write FFitDirection;
    property FitPercent:real read GetFitPercent write SetFitPercent;
    property Mask:String read FMask write FMask;
    property StrValue:string read GetStrValue write SetStrValue;
  end;

  TStringSemanticalObject = class(TCustomSemanticalObject)
  private
    FBound:integer;
  protected
    FStrValue:string;
    function GetStrValue:String;override;
    procedure SetStrValue(Value:String);override;
    function GetFitPercent:real;override;
    procedure SetFitPercent(Value:real);override;
  published
    function FitChar(Value:Char):boolean;override;
    property StrValue:string read FStrValue write SetStrValue;
  end;

  TCharArraySemanticalObject = class(TStringSemanticalObject)
  private
    FItemIndex:integer;
  protected
    procedure SetStrValue(Value:String);override;
    function GetFitPercent:real;override;
    procedure SetFitPercent(Value:real);override;
  published
    function FitChar(Value:Char):boolean;override;
    property FitPercent:real read GetFitPercent write SetFitPercent;
    property ItemIndex:integer read FItemIndex write FItemIndex;
    property StrValue:string read FStrValue write SetStrValue;
  end;

  TSemanticalFit = class(TObjectList)
  published
    function FitChar(Value:Char):boolean;
    procedure SetFitDirection(Value:integer);
  end;

const
  CSpace = [' '];
  CTab = [chr(9)];
  CQuote = [chr(39)];
  CDocLineDelims = [chr(13),chr(10),chr(0)];
  CPunctuationDelims = [',',';','.',':','?','!'];
  CDocDelimiters = CPunctuationDelims + CSpace + CTab + CQuote + CDocLineDelims;
  CBrackets = ['(',')','[',']'];
  CProgLineDelims = [';'];
  CProgCommentDelims = ['{','}','/','*','(',')','.'];

  CHuAlpha = ['a','','b','c','d','e','','f','g','h','i','','j','k','l','m','n','o','','','','p','q','r','s','t','u','','','','v','w','x','y','z',
              'A','','B','C','D','E','','F','G','H','I','','J','K','L','M','N','O','','','','P','Q','R','S','T','U','','','','V','W','X','Y','Z'];

  CHuLowCaseAlphas = ['a','','b','c','d','e','','f','g','h','i','','j','k','l','m','n','o','','','','p','q','r','s','t','u','','','','v','w','x','y','z'];
  CHuUpCaseAlphas =  ['A','','B','C','D','E','','F','G','H','I','','J','K','L','M','N','O','','','','P','Q','R','S','T','U','','','','V','W','X','Y','Z'];

  CNumbers = ['1','2','3','4','5','6','7','8','9','0'];

  CHuLowCase = 'abcdefghijklmnopqrstuvwxyz';
  CHuUpCase =  'ABCDEFGHIJKLMNOPQRSTUVWXYZ';

  CSymbols = ['$','#','&','@','^','_'];

  CValidIdent = CHuAlpha + CNumbers + CSymbols;

  CMathChars = ['+','-','*','/','=','<','>'];

  CProgCodeChars = CMathChars + CValidIdent + CProgCommentDelims + CDocDelimiters + CBrackets;

  COtherChars = [chr(0)..chr(255)] - CProgCodeChars;

  CGreaterThen  =  1;
  CSmallerThen  = -1;
  CTotalEgale        =  0;
  CPartialEgaleLongerThen  =  2;
  CPartialEgaleShorterThen =  -2;
  CNotFound = -128;

  function FillStr(const iCount:cardinal;const Value:string):string;
  function HuLowChar(AChar:char):char;
  function HuUpChar(AChar:char):char;

implementation


{ TSemanticalFit }
function TSemanticalFit.FitChar(Value: Char): boolean;
var cv:integer;
begin
  Result:=False;
  for cv:=0 to Count-1 do
    Result:=Boolean(Max(Integer(Result),
        Integer(TCustomSemanticalObject(Items[cv]).FitChar(Value))));
end;

procedure TSemanticalFit.SetFitDirection(Value: integer);
var cv:integer;
begin
  for cv:=0 to Count-1 do
    TCustomSemanticalObject(Items[cv]).FitDirection:=Value;
end;

{ TStringSemanticObject }
function TStringSemanticalObject.FitChar(Value: Char): boolean;
begin
  if ((FMask[FBound+FFitDirection]='1') and (StrValue[FBound+FFitDirection]=Value))
   or((FMask[FBound+FFitDirection]='2') and (HuLowChar(StrValue[FBound+FFitDirection])=HuLowChar(Value)))
   or (FMask[FBound+FFitDirection]='?') then
    FBound:=FBound+FFitDirection
  else
    FitPercent:=0;

  result:=((FFitDirection>0) and (FBound=Length(StrValue)))
       or ((FFitDirection<0) and (FBound=1));
end;

function TStringSemanticalObject.GetFitPercent: real;
begin
  try
    result:=Abs((Integer(FFitDirection<0)-
      ((FBound-Integer(FFitDirection<0))/Length(StrValue)))
        *100)
  except
    result:=Integer(FFitDirection<0)*100;
  end
end;

function TStringSemanticalObject.GetStrValue: String;
begin
  result:=FStrValue;
end;

procedure TStringSemanticalObject.SetFitPercent(Value: real);
begin
  if FitDirection>0 then
    FBound:=0
  else
    FBound:=Length(StrValue)+1
end;

procedure TStringSemanticalObject.SetStrValue(Value: String);
begin
  FStrValue:=Value;
  FitPercent:=0;
End;
{ TCharArraySemanticObject }

function TCharArraySemanticalObject.FitChar(Value: Char): boolean;
begin
  FItemIndex:=Pos(Value,FStrValue);
  Result:=(FItemIndex>0) and (FMask[FItemIndex]='1');
end;

function TCharArraySemanticalObject.GetFitPercent: real;
begin
  Result:=Integer(FItemIndex>0)*100;
end;

procedure TCharArraySemanticalObject.SetFitPercent(Value: real);
begin
  FItemIndex:=0;
end;

procedure TCharArraySemanticalObject.SetStrValue(Value: String);
begin
  FStrValue:=Value;
  FItemIndex:=0;
end;

//******************************************************************************
// 2000, Papp Zoltan.
// String sorminta generalasa
//
function FillStr(const iCount:cardinal;const Value:string):string;
var i:cardinal;
Begin
  Result:='';
  for i:=1 to iCount do
    Result:=Result+Value;
End;
//------------------------------------------------------------------------------

//******************************************************************************
// 2000, Papp Zoltan.
// Magyar kisbetuve alakitas
//
function HuLowChar(AChar:char):char;
var cv:integer;
    lvFound:boolean;
Begin
  lvFound:=False;
  for cv:=1 to length(CHuUpCase) do
    if AChar=CHuUpCase[cv] then
    Begin
      lvFound:=True;
      break;
    End;
  if lvFound then
    Result:=CHuLowCase[cv]
  else
    Result:=AChar
End;
//------------------------------------------------------------------------------

//******************************************************************************
// 2000, Papp Zoltan.
// Magyar nagybetuve alakitas
//
function HuUpChar(AChar:char):char;
var cv:integer;
    lvFound:boolean;
Begin
  lvFound:=False;
  for cv:=1 to length(CHuLowCase) do
    if AChar=CHuLowCase[cv] then
    Begin
      lvFound:=True;
      break;
    End;
  if lvFound then
    Result:=CHuUpCase[cv]
  else
    Result:=AChar
End;
//------------------------------------------------------------------------------

end.
